==> dmail-1.14.tgz: dmail/sub.c <==

/*
 * SUB.C
 *
 *  Global Routines:    INDEXOF()
 *                      SIG()
 *                      POSITION_CURRENT()
 *                      SKIP_TO_DATE()
 *                      GET_FIELD()
 *                      COMPILE_FIELD()
 *                      ISFROM()
 *                      XSTRNCMP()
 *                      NEXT_WORD()
 *                      DONE()
 *
 */

#include "dmail.h"

#ifdef sgi
#define SENDMAIL "/usr/lib/sendmail"
#else
#define SENDMAIL "/usr/sbin/sendmail"
#endif

Prototype int xfree(void *ptr);
Prototype int indexof(int num);
Prototype void null(void);
Prototype const char *get_field(char *str, int *plen);
Prototype int xstrncmp(char *src, char *dest, int len);
Prototype char *next_word(char *str);
Prototype int done(int n);
Prototype void fix_globals(void);
Prototype int _pager(const char *str, int nl, int len);


int
xfree(void *ptr)
{
    static void *sptr;

    if (sptr)
        free (sptr);
    sptr = ptr;
    return (1);
}
    
int
indexof(int num)
{
    int i;
    int last = -1;
    static int li;

    if (num < 1)
        num = -1;
    if (li >= Entries)
	li = 0;

#ifdef NOTDEF
    for (i = 0; i < Entries; ++i) {
	if (Entry[i].no == num)
	    printf("match %d index %d\n", num, i);
    }
#endif

    for (i = li; i < Entries; ++i) {
        if (Entry[i].no) {
	    if (last < i)
		last = i;
            if (Entry[i].no == num) {
		li = i;
                return (i);
	    }
        }
    }
    for (i = li; i >= 0; --i) {
        if (Entry[i].no) {
	    if (last < i)
		last = i;
            if (Entry[i].no == num) {
		li = i;
                return (i);
	    }
        }
    }
    if (num == -1 && last >= 0)
        return (last);
    return (-1);
}

void
null(void)
{
}

const char *
get_field(char *str, int *plen)
{
    int i, entry = Current;
    int l;
    int len = strlen(str);
    int pos;

    *plen = 0;

    if (Current < 0)
        return("");
    i = get_extra (str);
    if (i >= 0) {
	*plen = data_line_length(Entry[entry].fields[i]) - 1;
	if (*plen <= 0) {
	    *plen = 0;
	    return("");
	}
        return (Entry[entry].fields[i]);
    }
    if (FileFd < 0)
        return ("");

    pos = Entry[entry].fpos;
    l = position_next(&pos);

    while (l > 0 && !position_at_from(pos)) {
	const char *p = FileData + pos;

	if (*p == '\n')
	    break;
	if (len <= l && strncasecmp(Buf, p, len) == 0) {
	    while (position_next(&pos) > 0) {
		if (position_at_from(pos))
		    break;
		if (FileData[pos] != ' ' && FileData[pos] != '\t')
		    break;
	    }
	    *plen = pos - (p - FileData);
	    if (*plen < 0) {
		*plen = 0;
		return("");
	    }
	    return(p);
	}
	l = position_next(&pos);
    }
    return ("");
}

int
xstrncmp(char *src, char *dest, int len)
{
    while (--len >= 0) {
        if ((*src & 0x1f) != (*dest & 0x1f)) {
            if ((*src & 0x1f) < (*dest & 0x1f))
                return (-1);
            return (1);
        }
        ++src; ++dest;
    }
    return (0);
}

char *
next_word(char *str)
{
    while (*str  &&  *str != ' '  && *str != 9)
        ++str;
    while (*str  &&  (*str == ' ' || *str == 9))
        ++str;
    return (str);
}

int
done(int n)
{
    char scr[64];

    push_break();
    sprintf (scr, "/var/tmp/dmail%d", (int)getpid());
    unlink (scr);
    sprintf (scr, "/var/tmp/dmt%d", (int)getpid());
    unlink (scr);
    unlink ("#");
    exit (n);
}

void
fix_globals(void)
{
    char *ptr;

    push_break();
    S_page = (ptr = get_var (LEVEL_SET, "page")) ? 
            ((*ptr) ? atoi (ptr) : 24) : -1;
    if (S_page > 0  && (S_page -= 4) < 0)
        S_page = 1;

    S_sendmail = (ptr = get_var (LEVEL_SET, "sendmail")) ? ptr : SENDMAIL;
    S_novibreak= (ptr = get_var (LEVEL_SET, "vibreak")) ? 0 : 1;
    S_verbose  = (ptr = get_var (LEVEL_SET, "verbose")) ? 1 : 0;
    S_ask      = (ptr = get_var (LEVEL_SET, "ask")) ? 1 : 0;
    S_archive  = (ptr = get_var (LEVEL_SET, "archive")) ? 1 : 0;
    if (S_archive && *ptr == '\0')
        S_archive = 0;
    pop_break();
}

int
_pager(const char *str, int nl, int len)
{
    static int count;
    static FILE *fi;
    char buf[1024];
    char *ptr;

    if (str == NULL) {
        switch (S_page) {
        case -1:
            count = 0;
            return (1);
        case 0:
            ptr = get_var (LEVEL_SET, "page");
            fi = popen (ptr, "w");
            if (fi == NULL) {
                count = 0;
                printf ("CANNOT RUN PAGER PROGRAM: %s\n", ptr);
            } else {
                count = -1;
            }
            return (1);
        default:
            count = 0;
            return (1);
        }
    }
    if ((long)str == -1) {
        if (fi != NULL) {
            pclose (fi);
            fi = NULL;
        }
        return (1);
    }
    if (len < 0)
	len = strlen(str);
    if (count < 0) {
	fwrite(str, len, 1, fi);
        while (nl--)
            fputs ("\n", fi);
    } else {
	fwrite(str, len, 1, stdout);
        while (nl--) {
            fputs ("\n", stdout);
            ++count;
        }
        while (*str && len) {
            if (*str++ == '\n')
                ++count;
	    --len;
        }
        if (S_page > 0 && S_page <= count) {
            count = 0;
            puts ("\n-- more --");
            fgets(buf, sizeof(buf), stdin);
        }
    }
    return(1);
}

==> dmail-1.18.tgz: dmail/sub.c <==

/*
 * SUB.C
 *
 *  Global Routines:    INDEXOF()
 *                      SIG()
 *                      POSITION_CURRENT()
 *                      SKIP_TO_DATE()
 *                      GET_FIELD()
 *                      COMPILE_FIELD()
 *                      ISFROM()
 *                      XSTRNCMP()
 *                      NEXT_WORD()
 *                      DONE()
 *
 */

#include "dmail.h"

#ifdef sgi
#define SENDMAIL "/usr/lib/sendmail"
#else
#define SENDMAIL "/usr/sbin/sendmail"
#endif

Prototype int xfree(void *ptr);
Prototype int indexof(int num);
Prototype void null(void);
Prototype const char *get_field(char *str, int *plen);
Prototype int xstrncmp(char *src, char *dest, int len);
Prototype char *next_word(char *str);
Prototype int done(int n);
Prototype void fix_globals(void);
Prototype int _pager(const char *str, int nl, int len);


int
xfree(void *ptr)
{
    static void *sptr;

    if (sptr)
        free (sptr);
    sptr = ptr;
    return (1);
}
    
int
indexof(int num)
{
    int i;
    int last = -1;
    static int li;

    if (num < 1)
        num = -1;
    if (li >= Entries)
	li = 0;

#ifdef NOTDEF
    for (i = 0; i < Entries; ++i) {
	if (Entry[i].no == num)
	    printf("match %d index %d\n", num, i);
    }
#endif

    for (i = li; i < Entries; ++i) {
        if (Entry[i].no) {
	    if (last < i)
		last = i;
            if (Entry[i].no == num) {
		li = i;
                return (i);
	    }
        }
    }
    for (i = li; i >= 0; --i) {
        if (Entry[i].no) {
	    if (last < i)
		last = i;
            if (Entry[i].no == num) {
		li = i;
                return (i);
	    }
        }
    }
    if (num == -1 && last >= 0)
        return (last);
    return (-1);
}

void
null(void)
{
}

const char *
get_field(char *str, int *plen)
{
    int i, entry = Current;
    int l;
    int len = strlen(str);
    int pos;

    *plen = 0;

    if (Current < 0)
        return("");
    i = get_extra (str);
    if (i >= 0) {
	*plen = data_line_length(Entry[entry].fields[i], 1) - 1;
	if (*plen <= 0) {
	    *plen = 0;
	    return("");
	}
        return (Entry[entry].fields[i]);
    }
    if (FileFd < 0)
        return ("");

    pos = Entry[entry].fpos;
    l = position_next(&pos);

    while (l > 0 && !position_at_from(pos)) {
	const char *p = FileData + pos;

	if (*p == '\n')
	    break;
	if (len <= l && strncasecmp(Buf, p, len) == 0) {
	    while (position_next(&pos) > 0) {
		if (position_at_from(pos))
		    break;
		if (FileData[pos] != ' ' && FileData[pos] != '\t')
		    break;
	    }
	    *plen = pos - (p - FileData);
	    if (*plen < 0) {
		*plen = 0;
		return("");
	    }
	    return(p);
	}
	l = position_next(&pos);
    }
    return ("");
}

int
xstrncmp(char *src, char *dest, int len)
{
    while (--len >= 0) {
        if ((*src & 0x1f) != (*dest & 0x1f)) {
            if ((*src & 0x1f) < (*dest & 0x1f))
                return (-1);
            return (1);
        }
        ++src; ++dest;
    }
    return (0);
}

char *
next_word(char *str)
{
    while (*str  &&  *str != ' '  && *str != 9)
        ++str;
    while (*str  &&  (*str == ' ' || *str == 9))
        ++str;
    return (str);
}

int
done(int n)
{
    char scr[64];

    push_break();
    sprintf (scr, "/var/tmp/dmail%d", (int)getpid());
    unlink (scr);
    sprintf (scr, "/var/tmp/dmt%d", (int)getpid());
    unlink (scr);
    unlink ("#");
    exit (n);
}

void
fix_globals(void)
{
    char *ptr;

    push_break();
    S_page = (ptr = get_var (LEVEL_SET, "page")) ? 
            ((*ptr) ? atoi (ptr) : 24) : -1;
    if (S_page > 0  && (S_page -= 4) < 0)
        S_page = 1;

    S_sendmail = (ptr = get_var (LEVEL_SET, "sendmail")) ? ptr : SENDMAIL;
    S_novibreak= (ptr = get_var (LEVEL_SET, "vibreak")) ? 0 : 1;
    S_verbose  = (ptr = get_var (LEVEL_SET, "verbose")) ? 1 : 0;
    S_ask      = (ptr = get_var (LEVEL_SET, "ask")) ? 1 : 0;
    S_archive  = (ptr = get_var (LEVEL_SET, "archive")) ? 1 : 0;
    if (S_archive && *ptr == '\0')
        S_archive = 0;
    S_quiet    = (ptr = get_var (LEVEL_SET, "quiet")) ? 1 : 0;
    pop_break();
}

int
_pager(const char *str, int nl, int len)
{
    static int count;
    static FILE *fi;
    char buf[1024];
    char *ptr;

    if (str == NULL) {
        switch (S_page) {
        case -1:
            count = 0;
            return (1);
        case 0:
            ptr = get_var (LEVEL_SET, "page");
            fi = popen (ptr, "w");
            if (fi == NULL) {
                count = 0;
                printf ("CANNOT RUN PAGER PROGRAM: %s\n", ptr);
            } else {
                count = -1;
            }
            return (1);
        default:
            count = 0;
            return (1);
        }
    }
    if ((long)str == -1) {
        if (fi != NULL) {
            pclose (fi);
            fi = NULL;
        }
        return (1);
    }
    if (len < 0)
	len = strlen(str);
    if (count < 0) {
	fwrite(str, len, 1, fi);
        while (nl--)
            fputs ("\n", fi);
    } else {
	fwrite(str, len, 1, stdout);
        while (nl--) {
            fputs ("\n", stdout);
            ++count;
        }
        while (*str && len) {
            if (*str++ == '\n')
                ++count;
	    --len;
        }
        if (S_page > 0 && S_page <= count) {
            count = 0;
            puts ("\n-- more --");
            fgets(buf, sizeof(buf), stdin);
        }
    }
    return(1);
}
